var SGBD = require(__dirname+'/../sgbd/index');
var Table = require(__dirname+'/Table');

module.exports = function($conf){
    //extend from sgbd
    SGBD.apply(this, arguments);

    $conf = $conf || {};
    $conf.stringifyObjects = true;

    var self = this;

    var mysql      =  require('mysql');

    var $connection = mysql.createConnection($conf);

    $connection.config.queryFormat = function (query, values) {
        if (!values) return query;
        return query.replace(/\:(\w+)/g, function (txt, key) {
            if (values.hasOwnProperty(key)) {
                return this.escape(values[key]);
            }
            return txt;
        }.bind(this));
    };

    this.destroy = function(){
        return $connection.destroy();
    };

    this.connect = function(){
        return new Promise(function(resolve, reject){
            $connection.connect(function(err){
                if(err) reject(err);
                else resolve();
            });
        });
    };

    this.disconnect = function(){
        return new Promise(function(resolve, reject){
            $connection.end(function(err){
                if(err) reject(err);
                else resolve();
            });
        });
    };

    this.use = function(name){
        return new Promise(function(resolve, reject){
            $connection.query('USE '+name, function(err, result){
               if(err){
                   reject(err);
               } else{
                   resolve(result);
               }
            });
        });
    };

    this.database = function(name, charset, collate){
        var query = [];
        if(charset) query.push('CHARACTER SET '+charset);
        if(collate) query.push('COLLATE '+collate);

        return new Promise(function(resolve, reject){
            $connection.query('CREATE DATABASE IF NOT EXISTS '+name+' '+query.join(' '), function(err, result){
                if(err){
                    reject(err);
                }else{
                    self.use(name).then(function(){
                        // self.disconnect();
                        resolve(result);
                    });
                }
            });
        })
    };

    this.query = function(query, params, call){
        var cb = function(err, rsp){
            if(call) call(err, rsp);
            // self.disconnect();
        };

        return  this.connect().then(function(){
            if(typeof params == 'function'){
                call = params;
                $connection.query(query, cb);
            }else{
                $connection.query(query, params, cb);
            }
        });
    };

    this.tables = function(){
        return new Promise(function(resolve, reject){
            $connection.query('SHOW TABLES', function(err, result){
                if(err) reject(err);
                else{
                    var tables = [];
                    for(var i in result){
                        for(var j in result[i]){
                            tables.push(result[i][j]);
                            break;
                        }
                    }
                    resolve(tables);
                }
            });
        });
    };

    this.table = function(name){
        var table = new Table(name, $connection);
        table.$sql = this;
        return table;
    };

};